@@ -144,6 +144,7 @@ typedef enum {
 	/* Convenience field for getting a name from the contact.
 	   Returns the first one of [File-As, Full Name, Org, Email1]
 	   to be set */
+	E_CONTACT_REV,     /* string field to hold  time of last update to this vcard*/
 	E_CONTACT_NAME_OR_ORG,
 
 	/* Address fields */
@@ -184,7 +185,9 @@ typedef enum {
 	/* Security Fields */
 	E_CONTACT_X509_CERT,     /* structured field (EContactCert) */
 
+
 	E_CONTACT_FIELD_LAST,
+	E_CONTACT_FIELD_FIRST        = E_CONTACT_UID,
 
 	/* useful constants */
 	E_CONTACT_LAST_SIMPLE_STRING = E_CONTACT_NAME_OR_ORG,
@@ -203,7 +206,7 @@ typedef struct {
 	char *family;
 	char *given;
 	char *additional;
-	char *prefixes;
+ 	char *prefixes;
 	char *suffixes;
 } EContactName;
 
