@@ -25,9 +25,12 @@
 
 #include <string.h>
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 #include <bonobo/bonobo-exception.h>
 #include <bonobo/bonobo-moniker-util.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n-lib.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libedataserver/e-util.h>
 #include <libedataserver/e-xml-hash-utils.h>
@@ -55,6 +58,10 @@ struct _ECalBackendFilePrivate {
 	/* Filename in the dir */
 	char *file_name;	
 	gboolean read_only;
+	gboolean is_dirty;
+	guint dirty_idle_id;
+
+	GMutex *idle_save_mutex;
 
 	/* Toplevel VCALENDAR component */
 	icalcomponent *icalcomp;
@@ -87,25 +94,14 @@ static ECalBackendSyncClass *parent_class;
 
 
 
-/* g_hash_table_foreach() callback to destroy recurrences in the hash table */
-static void
-free_recurrence (gpointer key, gpointer value, gpointer data)
-{
-	char *rid = key;
-	ECalComponent *comp = value;
-
-	g_free (rid);
-	g_object_unref (comp);
-}
-
 /* g_hash_table_foreach() callback to destroy a ECalBackendFileObject */
 static void
-free_object (gpointer key, gpointer value, gpointer data)
+free_object_data (gpointer data)
 {
-	ECalBackendFileObject *obj_data = value;
+	ECalBackendFileObject *obj_data = data;
 
-	g_object_unref (obj_data->full_object);
-	g_hash_table_foreach (obj_data->recurrences, (GHFunc) free_recurrence, NULL);
+	if (obj_data->full_object)
+		g_object_unref (obj_data->full_object);
 	g_hash_table_destroy (obj_data->recurrences);
 	g_list_free (obj_data->recurrences_list);
 
@@ -113,8 +109,8 @@ free_object (gpointer key, gpointer value, gpointer data)
 }
 
 /* Saves the calendar data */
-static void
-save (ECalBackendFile *cbfile)
+static gboolean
+save_file_when_idle (gpointer user_data)
 {
 	ECalBackendFilePrivate *priv;
 	GnomeVFSURI *uri, *backup_uri;
@@ -123,11 +119,19 @@ save (ECalBackendFile *cbfile)
 	GnomeVFSFileSize out;
 	gchar *tmp, *backup_uristr;
 	char *buf;
+	ECalBackendFile *cbfile = user_data;
 	
 	priv = cbfile->priv;
 	g_assert (priv->uri != NULL);
 	g_assert (priv->icalcomp != NULL);
 
+	g_mutex_lock (priv->idle_save_mutex);
+	if (!priv->is_dirty) {
+		priv->dirty_idle_id = 0;
+		g_mutex_unlock (priv->idle_save_mutex);
+		return FALSE;
+	}
+
 	uri = gnome_vfs_uri_new (priv->uri);
 	if (!uri)
 		goto error_malformed_uri;
@@ -176,29 +180,49 @@ save (ECalBackendFile *cbfile)
 	if (result != GNOME_VFS_OK)
 		goto error;
 
-	return;
+	priv->is_dirty = FALSE;
+	priv->dirty_idle_id = 0;
+
+	g_mutex_unlock (priv->idle_save_mutex);
+
+	return FALSE;
 
  error_malformed_uri:
+	g_mutex_unlock (priv->idle_save_mutex);
 	e_cal_backend_notify_error (E_CAL_BACKEND (cbfile),
 				  _("Can't save calendar data: Malformed URI."));
-	return;
+	return TRUE;
 
  error:
+	g_mutex_unlock (priv->idle_save_mutex);
 	e_cal_backend_notify_error (E_CAL_BACKEND (cbfile), gnome_vfs_result_to_string (result));
-	return;
+	return TRUE;
+}
+
+static void
+save (ECalBackendFile *cbfile)
+{
+	ECalBackendFilePrivate *priv;
+
+	priv = cbfile->priv;
+
+	g_mutex_lock (priv->idle_save_mutex);
+	priv->is_dirty = TRUE;
+
+	if (!priv->dirty_idle_id)
+		priv->dirty_idle_id = g_idle_add ((GSourceFunc) save_file_when_idle, cbfile);
+
+	g_mutex_unlock (priv->idle_save_mutex);
 }
 
 static void
 free_calendar_components (GHashTable *comp_uid_hash, icalcomponent *top_icomp)
 {
-	if (comp_uid_hash) {
-		g_hash_table_foreach (comp_uid_hash, (GHFunc) free_object, NULL);
+	if (comp_uid_hash)
 		g_hash_table_destroy (comp_uid_hash);
-	}
 
-	if (top_icomp) {
+	if (top_icomp)
 		icalcomponent_free (top_icomp);
-	}
 }
 
 static void
@@ -227,6 +251,8 @@ e_cal_backend_file_dispose (GObject *object)
 	priv = cbfile->priv;
 
 	/* Save if necessary */
+	if (priv->is_dirty)
+		save_file_when_idle (cbfile);
 
 	free_calendar_data (cbfile);
 
@@ -249,6 +275,16 @@ e_cal_backend_file_finalize (GObject *object)
 
 	/* Clean up */
 
+	if (priv->dirty_idle_id) {
+		g_source_remove (priv->dirty_idle_id);
+		priv->dirty_idle_id = 0;
+	}
+
+	if (priv->idle_save_mutex) {
+		g_mutex_free (priv->idle_save_mutex);
+		priv->idle_save_mutex = NULL;
+	}
+
 	if (priv->uri) {
 	        g_free (priv->uri);
 		priv->uri = NULL;
@@ -399,7 +435,6 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 	ECalBackendFilePrivate *priv;
 	ECalBackendFileObject *obj_data;
 	const char *uid;
-	GSList *categories;
 
 	priv = cbfile->priv;
 
@@ -417,8 +452,8 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 		} else {
 			obj_data = g_new0 (ECalBackendFileObject, 1);
 			obj_data->full_object = NULL;
-			obj_data->recurrences = g_hash_table_new (g_str_hash, g_str_equal);
-			g_hash_table_insert (priv->comp_uid_hash, (gpointer) uid, obj_data);
+			obj_data->recurrences = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
+			g_hash_table_insert (priv->comp_uid_hash, g_strdup (uid), obj_data);
 		}
 
 		g_hash_table_insert (obj_data->recurrences, g_strdup (rid), comp);
@@ -439,9 +474,9 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 		} else {
 			obj_data = g_new0 (ECalBackendFileObject, 1);
 			obj_data->full_object = comp;
-			obj_data->recurrences = g_hash_table_new (g_str_hash, g_str_equal);
+			obj_data->recurrences = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
 
-			g_hash_table_insert (priv->comp_uid_hash, (gpointer) uid, obj_data);
+			g_hash_table_insert (priv->comp_uid_hash, g_strdup (uid), obj_data);
 		}
 	}
 
@@ -456,20 +491,16 @@ add_component (ECalBackendFile *cbfile, ECalComponent *comp, gboolean add_to_top
 		g_assert (icalcomp != NULL);
 
 		icalcomponent_add_component (priv->icalcomp, icalcomp);
-	}
 
-	/* Update the set of categories */
-	e_cal_component_get_categories_list (comp, &categories);
-	e_cal_backend_ref_categories (E_CAL_BACKEND (cbfile), categories);
-	e_cal_component_free_categories_list (categories);
+		save (cbfile);
+	}
 }
 
-/* g_hash_table_foreach() callback to remove recurrences from the calendar */
-static void
+/* g_hash_table_foreach_remove() callback to remove recurrences from the calendar */
+static gboolean
 remove_recurrence_cb (gpointer key, gpointer value, gpointer data)
 {
 	GList *l;
-	GSList *categories;
 	icalcomponent *icalcomp;
 	ECalBackendFilePrivate *priv;
 	ECalComponent *comp = value;
@@ -487,10 +518,7 @@ remove_recurrence_cb (gpointer key, gpointer value, gpointer data)
 	l = g_list_find (priv->comp, comp);
 	priv->comp = g_list_delete_link (priv->comp, l);
 
-	/* update the set of categories */
-	e_cal_component_get_categories_list (comp, &categories);
-	e_cal_backend_unref_categories (E_CAL_BACKEND (cbfile), categories);
-	e_cal_component_free_categories_list (categories);
+	return TRUE;
 }
 
 /* Removes a component from the backend's hash and lists.  Does not perform
@@ -498,46 +526,33 @@ remove_recurrence_cb (gpointer key, gpointer value, gpointer data)
  * icalcomponent.
  */
 static void
-remove_component (ECalBackendFile *cbfile, ECalComponent *comp)
+remove_component (ECalBackendFile *cbfile, const char *uid, ECalBackendFileObject *obj_data)
 {
 	ECalBackendFilePrivate *priv;
 	icalcomponent *icalcomp;
-	const char *uid;
 	GList *l;
-	GSList *categories;
-	ECalBackendFileObject *obj_data;
 
 	priv = cbfile->priv;
 
 	/* Remove the icalcomp from the toplevel */
+	if (obj_data->full_object) {
+		icalcomp = e_cal_component_get_icalcomponent (obj_data->full_object);
+		g_assert (icalcomp != NULL);
 
-	icalcomp = e_cal_component_get_icalcomponent (comp);
-	g_assert (icalcomp != NULL);
+		icalcomponent_remove_component (priv->icalcomp, icalcomp);
 
-	icalcomponent_remove_component (priv->icalcomp, icalcomp);
+		/* Remove it from our mapping */
+		l = g_list_find (priv->comp, obj_data->full_object);
+		g_assert (l != NULL);
+		priv->comp = g_list_delete_link (priv->comp, l);
+	}
 
-	/* Remove it from our mapping */
-
-	e_cal_component_get_uid (comp, &uid);
-	obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
-	if (!obj_data)
-		return;
+	/* remove the recurrences also */
+	g_hash_table_foreach_remove (obj_data->recurrences, (GHRFunc) remove_recurrence_cb, cbfile);
 
 	g_hash_table_remove (priv->comp_uid_hash, uid);
 
-	l = g_list_find (priv->comp, comp);
-	g_assert (l != NULL);
-	priv->comp = g_list_delete_link (priv->comp, l);
-
-	/* remove the recurrences also */
-	g_hash_table_foreach (obj_data->recurrences, (GHFunc) remove_recurrence_cb, cbfile);
-
-	/* Update the set of categories */
-	e_cal_component_get_categories_list (comp, &categories);
-	e_cal_backend_unref_categories (E_CAL_BACKEND (cbfile), categories);
-	e_cal_component_free_categories_list (categories);
-
-	free_object ((gpointer) uid, (gpointer) obj_data, NULL);
+	save (cbfile);
 }
 
 /* Scans the toplevel VCALENDAR component and stores the objects it finds */
@@ -647,7 +662,7 @@ open_cal (ECalBackendFile *cbfile, const char *uristr)
 	priv->icalcomp = icalcomp;
 	priv->uri = get_uri_string_for_gnome_vfs (E_CAL_BACKEND (cbfile));
 
-	priv->comp_uid_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	priv->comp_uid_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, free_object_data);
 	scan_vcalendar (cbfile);
 
 	return GNOME_Evolution_Calendar_Success;
@@ -682,7 +697,7 @@ notify_removals_cb (gpointer key, gpointer value, gpointer data)
 		if (!old_obj_str)
 			return;
 
-		e_cal_backend_notify_object_removed (context->backend, uid, old_obj_str);
+		e_cal_backend_notify_object_removed (context->backend, uid, old_obj_str, NULL);
 	}
 }
 
@@ -779,7 +794,7 @@ reload_cal (ECalBackendFile *cbfile, const char *uristr)
 
 	priv->icalcomp = icalcomp;
 
-	priv->comp_uid_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	priv->comp_uid_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, free_object_data);
 	scan_vcalendar (cbfile);
 
 	priv->uri = get_uri_string_for_gnome_vfs (E_CAL_BACKEND (cbfile));
@@ -815,7 +830,7 @@ create_cal (ECalBackendFile *cbfile, const char *uristr)
 	priv->icalcomp = e_cal_util_new_top_level ();
 
 	/* Create our internal data */
-	priv->comp_uid_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	priv->comp_uid_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, free_object_data);
 
 	priv->uri = get_uri_string_for_gnome_vfs (E_CAL_BACKEND (cbfile));
 
@@ -1237,6 +1252,8 @@ e_cal_backend_file_get_object_list (ECalBackendSync *backend, EDataCal *cal, con
 	g_hash_table_foreach (priv->comp_uid_hash, (GHFunc) match_object_sexp, &match_data);
 
 	*objects = match_data.obj_list;
+
+	g_object_unref (match_data.obj_sexp);
 	
 	return GNOME_Evolution_Calendar_Success;	
 }
@@ -1280,6 +1297,7 @@ e_cal_backend_file_start_query (ECalBackend *backend, EDataCalView *query)
 		g_list_foreach (match_data.obj_list, (GFunc) g_free, NULL);
 		g_list_free (match_data.obj_list);
 	}
+	g_object_unref (match_data.obj_sexp);
 
 	e_data_cal_view_notify_done (query, GNOME_Evolution_Calendar_Success);
 }
@@ -1388,6 +1406,7 @@ create_user_free_busy (ECalBackendFile *cbfile, const char *address, const char
 						vcalendar_comp,
 						priv->default_zone);
 	}
+	g_object_unref (obj_sexp);
 
 	return vfb;	
 }
@@ -1644,7 +1663,6 @@ sanitize_component (ECalBackendFile *cbfile, ECalComponent *comp)
 
 }	
 
-
 static ECalBackendSyncStatus
 e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, char **calobj, char **uid)
 {
@@ -1721,10 +1739,45 @@ e_cal_backend_file_create_object (ECalBackendSync *backend, EDataCal *cal, char
 	return GNOME_Evolution_Calendar_Success;
 }
 
+typedef struct {
+	ECalBackendFile *cbfile;
+	ECalBackendFileObject *obj_data;
+	const char *rid;
+	CalObjModType mod;
+} RemoveRecurrenceData;
+
+static gboolean
+remove_object_instance_cb (gpointer key, gpointer value, gpointer user_data)
+{
+	time_t fromtt, instancett;
+	ECalComponent *instance = value;
+	RemoveRecurrenceData *rrdata = user_data;
+
+	fromtt = icaltime_as_timet (icaltime_from_string (rrdata->rid));
+	instancett = icaltime_as_timet (get_rid_icaltime (instance));
+
+	if (fromtt > 0 && instancett > 0) {
+		if ((rrdata->mod == CALOBJ_MOD_THISANDPRIOR && instancett <= fromtt) ||
+		    (rrdata->mod == CALOBJ_MOD_THISANDFUTURE && instancett >= fromtt)) {
+			/* remove the component from our data */
+			icalcomponent_remove_component (rrdata->cbfile->priv->icalcomp,
+							e_cal_component_get_icalcomponent (instance));
+			rrdata->cbfile->priv->comp = g_list_remove (rrdata->cbfile->priv->comp, instance);
+
+			rrdata->obj_data->recurrences_list = g_list_remove (rrdata->obj_data->recurrences_list, instance);
+
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static ECalBackendSyncStatus
 e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const char *calobj, 
 				  CalObjModType mod, char **old_object)
 {
+	RemoveRecurrenceData rrdata;
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
 	icalcomponent *icalcomp;
@@ -1797,8 +1850,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 			return GNOME_Evolution_Calendar_Success;
 		}
 
-		if (g_hash_table_lookup_extended (obj_data->recurrences, rid,
-						  (void **) &real_rid, (void **) &recurrence)) {
+		if (g_hash_table_lookup_extended (obj_data->recurrences, rid, &real_rid, &recurrence)) {
 			if (old_object)
 				*old_object = e_cal_component_get_as_string (recurrence);
 
@@ -1806,12 +1858,8 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 			icalcomponent_remove_component (priv->icalcomp,
 							e_cal_component_get_icalcomponent (recurrence));
 			priv->comp = g_list_remove (priv->comp, recurrence);
-			g_hash_table_remove (obj_data->recurrences, rid);
 			obj_data->recurrences_list = g_list_remove (obj_data->recurrences_list, recurrence);
-
-			/* free memory */
-			g_free (real_rid);
-			g_object_unref (recurrence);
+			g_hash_table_remove (obj_data->recurrences, rid);
 		} else {
 			char *old, *new;
 
@@ -1820,9 +1868,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 			e_cal_util_remove_instances (e_cal_component_get_icalcomponent (obj_data->full_object),
 						     get_rid_icaltime (comp),
 						     mod);
-
-			new = e_cal_component_get_as_string (obj_data->full_object);
-
+			new = e_cal_component_get_as_string (comp);
 			e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old, new);
 
 			if (old_object)
@@ -1833,14 +1879,77 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 		}
 
 		/* add the detached instance */
+		icalcomponent_set_recurrenceid (e_cal_component_get_icalcomponent (comp),
+						icaltime_from_string (rid));
 		g_hash_table_insert (obj_data->recurrences, 
 				     g_strdup (e_cal_component_get_recurid_as_string (comp)),
 				     comp);
+		icalcomponent_add_component (priv->icalcomp,
+					     e_cal_component_get_icalcomponent (comp));
+		priv->comp = g_list_append (priv->comp, comp);
 		obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, comp);
 		break;
 	case CALOBJ_MOD_THISANDPRIOR :
-		break;
 	case CALOBJ_MOD_THISANDFUTURE :
+		rid = e_cal_component_get_recurid_as_string (comp);
+		if (!rid || !*rid) {
+			if (old_object)
+				*old_object = e_cal_component_get_as_string (obj_data->full_object);
+
+			remove_component (cbfile, comp_uid, obj_data);
+
+			/* Add the new object */
+			add_component (cbfile, comp, TRUE);
+			break;
+		}
+
+		/* remove the component from our data, temporarily */
+		icalcomponent_remove_component (priv->icalcomp,
+						e_cal_component_get_icalcomponent (obj_data->full_object));
+		priv->comp = g_list_remove (priv->comp, obj_data->full_object);
+
+		/* now deal with the detached recurrence */
+		if (g_hash_table_lookup_extended (obj_data->recurrences, rid,
+						  &real_rid, &recurrence)) {
+			if (old_object)
+				*old_object = e_cal_component_get_as_string (recurrence);
+
+			/* remove the component from our data */
+			icalcomponent_remove_component (priv->icalcomp,
+							e_cal_component_get_icalcomponent (recurrence));
+			priv->comp = g_list_remove (priv->comp, recurrence);
+			obj_data->recurrences_list = g_list_remove (obj_data->recurrences_list, recurrence);
+			g_hash_table_remove (obj_data->recurrences, rid);
+		} else {
+			if (old_object)
+				*old_object = e_cal_component_get_as_string (obj_data->full_object);
+		}
+
+		/* remove all affected recurrences */
+		e_cal_util_remove_instances (e_cal_component_get_icalcomponent (obj_data->full_object),
+					     icaltime_from_string (rid), mod);
+
+		rrdata.cbfile = cbfile;
+		rrdata.obj_data = obj_data;
+		rrdata.rid = rid;
+		rrdata.mod = mod;
+		g_hash_table_foreach_remove (obj_data->recurrences, (GHRFunc) remove_object_instance_cb, &rrdata);
+
+		/* add the modified object to the beginning of the list,
+		   so that it's always before any detached instance we
+		   might have */
+		icalcomponent_add_component (priv->icalcomp,
+					     e_cal_component_get_icalcomponent (obj_data->full_object));
+		priv->comp = g_list_prepend (priv->comp, obj_data->full_object);
+
+		/* add the new detached recurrence */
+		g_hash_table_insert (obj_data->recurrences, 
+				     g_strdup (e_cal_component_get_recurid_as_string (comp)),
+				     comp);
+		icalcomponent_add_component (priv->icalcomp,
+					     e_cal_component_get_icalcomponent (comp));
+		priv->comp = g_list_append (priv->comp, comp);
+		obj_data->recurrences_list = g_list_append (obj_data->recurrences_list, comp);
 		break;
 	case CALOBJ_MOD_ALL :
 		/* in this case, we blow away all recurrences, and start over
@@ -1849,7 +1958,7 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 		if (old_object)
 			*old_object = e_cal_component_get_as_string (obj_data->full_object);
 
-		remove_component (cbfile, obj_data->full_object);
+		remove_component (cbfile, comp_uid, obj_data);
 
 		/* Add the new object */
 		add_component (cbfile, comp, TRUE);
@@ -1866,27 +1975,17 @@ remove_instance (ECalBackendFile *cbfile, ECalBackendFileObject *obj_data, const
 {
 	char *hash_rid;
 	ECalComponent *comp;
-	GSList *categories;
 
 	if (!rid || !*rid)
 		return;
 
-	if (g_hash_table_lookup_extended (obj_data->recurrences, rid, (void **) &hash_rid, (void **) &comp)) {
+	if (g_hash_table_lookup_extended (obj_data->recurrences, rid, &hash_rid, &comp)) {
 		/* remove the component from our data */
 		icalcomponent_remove_component (cbfile->priv->icalcomp,
 						e_cal_component_get_icalcomponent (comp));
 		cbfile->priv->comp = g_list_remove (cbfile->priv->comp, comp);
-		g_hash_table_remove (obj_data->recurrences, rid);
 		obj_data->recurrences_list = g_list_remove (obj_data->recurrences_list, comp);
-
-		/* update the set of categories */
-		e_cal_component_get_categories_list (comp, &categories);
-		e_cal_backend_unref_categories (E_CAL_BACKEND (cbfile), categories);
-		e_cal_component_free_categories_list (categories);
-
-		/* free memory */
-		g_free (hash_rid);
-		g_object_unref (comp);
+		g_hash_table_remove (obj_data->recurrences, rid);
 
 		return;
 	}
@@ -1907,62 +2006,17 @@ remove_instance (ECalBackendFile *cbfile, ECalBackendFileObject *obj_data, const
 	cbfile->priv->comp = g_list_prepend (cbfile->priv->comp, obj_data->full_object);
 }
 
-typedef struct {
-	ECalBackendFile *cbfile;
-	ECalBackendFileObject *obj_data;
-	const char *rid;
-	CalObjModType mod;
-} RemoveRecurrenceData;
-
-static gboolean
-remove_object_instance_cb (gpointer key, gpointer value, gpointer user_data)
-{
-	time_t fromtt, instancett;
-	GSList *categories;
-	char *rid = key;
-	ECalComponent *instance = value;
-	RemoveRecurrenceData *rrdata = user_data;
-
-	fromtt = icaltime_as_timet (icaltime_from_string (rrdata->rid));
-	instancett = icaltime_as_timet (get_rid_icaltime (instance));
-
-	if (fromtt > 0 && instancett > 0) {
-		if ((rrdata->mod == CALOBJ_MOD_THISANDPRIOR && instancett <= fromtt) ||
-		    (rrdata->mod == CALOBJ_MOD_THISANDFUTURE && instancett >= fromtt)) {
-			/* remove the component from our data */
-			icalcomponent_remove_component (rrdata->cbfile->priv->icalcomp,
-							e_cal_component_get_icalcomponent (instance));
-			rrdata->cbfile->priv->comp = g_list_remove (rrdata->cbfile->priv->comp, instance);
-
-			rrdata->obj_data->recurrences_list = g_list_remove (rrdata->obj_data->recurrences_list, instance);
-
-			/* update the set of categories */
-			e_cal_component_get_categories_list (instance, &categories);
-			e_cal_backend_unref_categories (E_CAL_BACKEND (rrdata->cbfile), categories);
-			e_cal_component_free_categories_list (categories);
-
-			/* free memory */
-			g_free (rid);
-			g_object_unref (instance);
-
-			return TRUE;
-		}
-	}
-
-	return FALSE;
-}
-
 /* Remove_object handler for the file backend */
 static ECalBackendSyncStatus
 e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 				  const char *uid, const char *rid,
-				  CalObjModType mod, char **object)
+				  CalObjModType mod, char **old_object,
+				  char **object)
 {
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
 	ECalBackendFileObject *obj_data;
 	ECalComponent *comp;
-	GSList *categories;
 	RemoveRecurrenceData rrdata;
 
 	cbfile = E_CAL_BACKEND_FILE (backend);
@@ -1971,6 +2025,8 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	g_return_val_if_fail (priv->icalcomp != NULL, GNOME_Evolution_Calendar_NoSuchCal);
 	g_return_val_if_fail (uid != NULL, GNOME_Evolution_Calendar_ObjectNotFound);
 
+	*old_object = *object = NULL;
+
 	obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
 	if (!obj_data)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
@@ -1979,22 +2035,35 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 
 	switch (mod) {
 	case CALOBJ_MOD_ALL :
-		*object = e_cal_component_get_as_string (comp);
-		remove_component (cbfile, comp);
+		if (comp) {
+			*old_object = e_cal_component_get_as_string (comp);
+		} else {
+			char *real_rid;
+
+			if (g_hash_table_lookup_extended (obj_data->recurrences, rid, &real_rid, &comp))
+				*old_object = e_cal_component_get_as_string (comp);
+		}
+
+		remove_component (cbfile, uid, obj_data);
+
+		*object = NULL;
 		break;
 	case CALOBJ_MOD_THIS :
-		*object = e_cal_component_get_as_string (comp);
-		if (!rid || !*rid)
-			remove_component (cbfile, comp);
-		else
+		*old_object = e_cal_component_get_as_string (comp);
+		if (!rid || !*rid) {
+			remove_component (cbfile, uid, obj_data);
+			*object = NULL;
+		} else {
 			remove_instance (cbfile, obj_data, rid);
+			*object = e_cal_component_get_as_string (obj_data->full_object);
+		}
 		break;
 	case CALOBJ_MOD_THISANDPRIOR :
 	case CALOBJ_MOD_THISANDFUTURE :
 		if (!rid || !*rid)
 			return GNOME_Evolution_Calendar_ObjectNotFound;
 
-		*object = e_cal_component_get_as_string (comp);
+		*old_object = e_cal_component_get_as_string (comp);
 
 		/* remove the component from our data, temporarily */
 		icalcomponent_remove_component (priv->icalcomp,
@@ -2015,6 +2084,8 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 		   so that it's always before any detached instance we
 		   might have */
 		priv->comp = g_list_prepend (priv->comp, comp);
+
+		*object = e_cal_component_get_as_string (obj_data->full_object);
 		break;
 	}
 
@@ -2026,15 +2097,30 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 static gboolean
 cancel_received_object (ECalBackendFile *cbfile, icalcomponent *icalcomp)
 {
-	ECalComponent *old_comp;
+	ECalBackendFileObject *obj_data;
+	ECalBackendFilePrivate *priv;
+	const char *rid;
+	ECalComponent *comp;
+
+	priv = cbfile->priv;
 
 	/* Find the old version of the component. */
-	old_comp = lookup_component (cbfile, icalcomponent_get_uid (icalcomp));
-	if (!old_comp)
+	obj_data = g_hash_table_lookup (priv->comp_uid_hash, icalcomponent_get_uid (icalcomp));
+	if (!obj_data)
 		return FALSE;
 
 	/* And remove it */
-	remove_component (cbfile, old_comp);
+	comp = e_cal_component_new ();
+	if (!e_cal_component_set_icalcomponent (comp, icalcomponent_new_clone (icalcomp))) {
+		g_object_unref (comp);
+		return FALSE;
+	}
+
+	rid = e_cal_component_get_recurid_as_string (comp);
+	if (rid && *rid)
+		remove_instance (cbfile, obj_data, rid);
+	else
+		remove_component (cbfile, icalcomponent_get_uid (icalcomp), obj_data);
 
 	return TRUE;
 }
@@ -2056,6 +2142,80 @@ check_tzids (icalparameter *param, void *data)
 		tzdata->found = FALSE;
 }
 
+static void
+fetch_attachments (ECalBackendSync *backend, ECalComponent *comp)
+{
+	GSList *attach_list = NULL, *new_attach_list = NULL;
+	GSList *l;
+	char  *attach_store, *filename, *file_contents;
+	char *dest_url, *dest_file;
+	int fd, len;
+	int len_read = 0;
+	char buf[1024];
+	struct stat sb;
+	const char *uid;
+
+
+	e_cal_component_get_attachment_list (comp, &attach_list);
+	e_cal_component_get_uid (comp, &uid);
+	/*FIXME  get the uri rather than computing the path */
+	attach_store = g_strconcat (g_get_home_dir (), "/",
+			".evolution/calendar/local/system", NULL);
+	
+	for (l = attach_list; l ; l = l->next) {
+		char *sfname = (char *)l->data;
+
+		filename = g_strrstr (sfname, "/") + 1;	
+
+		// open the file using the data
+		fd = open (sfname, O_RDONLY); 
+		if (fd == -1) {
+			/* TODO handle error conditions */
+			g_message ("DEBUG: could not open the file descriptor\n");
+			continue;
+		}
+		if (fstat (fd, &sb) == -1) {
+			/* TODO handle error conditions */
+			g_message ("DEBUG: could not fstat the attachment file\n");
+			continue;
+		}
+		len = sb.st_size;
+
+		file_contents = g_malloc (len + 1);
+	
+		while (len_read < len) {
+			int c = read (fd, buf, sizeof (buf));
+
+			if (c == -1)
+				break;
+
+			memcpy (&file_contents[len_read], buf, c);
+			len_read += c;
+		}
+		file_contents [len_read] = 0;
+
+		/* write*/
+		dest_file = g_strconcat (attach_store, "/", uid, "-",
+				filename, NULL);
+		fd = open (dest_file, O_RDWR|O_CREAT|O_TRUNC, 0600);
+		if (fd == -1) {
+			/* TODO handle error conditions */
+			g_message ("DEBUG: could not serialize attachments\n");
+		}
+
+		if (write (fd, file_contents, len_read) == -1) {
+			/* TODO handle error condition */
+			g_message ("DEBUG: attachment write failed.\n");
+		}
+
+		dest_url = g_strconcat ("file:///", dest_file, NULL);
+		new_attach_list = g_slist_append (new_attach_list, dest_url);
+		g_free (dest_file);
+	}
+	g_free (attach_store);
+	e_cal_component_set_attachment_list (comp, new_attach_list);
+}
+
 /* Update_objects handler for the file backend. */
 static ECalBackendSyncStatus
 e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, const char *calobj)
@@ -2064,7 +2224,7 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 	ECalBackendFilePrivate *priv;
 	icalcomponent *toplevel_comp, *icalcomp = NULL;
 	icalcomponent_kind kind;
-	icalproperty_method method;
+	icalproperty_method toplevel_method, method;
 	icalcomponent *subcomp;
 	GList *comps, *del_comps, *l;
 	ECalComponent *comp;
@@ -2088,10 +2248,17 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		/* If its not a VCALENDAR, make it one to simplify below */
 		icalcomp = toplevel_comp;
 		toplevel_comp = e_cal_util_new_top_level ();
-		icalcomponent_add_component (toplevel_comp, icalcomp);	
+		if (icalcomponent_get_method (icalcomp) == ICAL_METHOD_CANCEL)
+			icalcomponent_set_method (toplevel_comp, ICAL_METHOD_CANCEL);
+		else
+			icalcomponent_set_method (toplevel_comp, ICAL_METHOD_PUBLISH);
+		icalcomponent_add_component (toplevel_comp, icalcomp);
+	} else {
+		if (!icalcomponent_get_first_property (toplevel_comp, ICAL_METHOD_PROPERTY))
+			icalcomponent_set_method (toplevel_comp, ICAL_METHOD_PUBLISH);
 	}
 
-	method = icalcomponent_get_method (toplevel_comp);
+	toplevel_method = icalcomponent_get_method (toplevel_comp);
 
 	/* Build a list of timezones so we can make sure all the objects have valid info */
 	tzdata.zones = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
@@ -2145,7 +2312,7 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 	for (l = comps; l; l = l->next) {
 		const char *uid, *rid;
 		char *object, *old_object;
-		ECalComponent *old_comp;
+		ECalBackendFileObject *obj_data;
 
 		subcomp = l->data;
 	
@@ -2161,14 +2328,25 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		e_cal_component_get_uid (comp, &uid);
 		rid = e_cal_component_get_recurid_as_string (comp);
 
+		if (icalcomponent_get_first_property (subcomp, ICAL_METHOD_PROPERTY))
+			method = icalcomponent_get_method (subcomp);
+		else
+			method = toplevel_method;
+
 		switch (method) {
 		case ICAL_METHOD_PUBLISH:
 		case ICAL_METHOD_REQUEST:
 		case ICAL_METHOD_REPLY:
-			old_comp = lookup_component (cbfile, uid);
-			if (old_comp) {
-				old_object = e_cal_component_get_as_string (old_comp);
-				remove_component (cbfile, old_comp);
+			/* handle attachments */
+			if (e_cal_component_has_attachments (comp))
+				fetch_attachments (backend, comp);
+			obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
+			if (obj_data) {
+				old_object = e_cal_component_get_as_string (obj_data->full_object);
+				if (rid)
+					remove_instance (cbfile, obj_data, rid);
+				else
+					remove_component (cbfile, uid, obj_data);
 				add_component (cbfile, comp, FALSE);
 
 				object = e_cal_component_get_as_string (comp);
@@ -2177,7 +2355,7 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 				g_free (old_object);
 			} else {
 				add_component (cbfile, comp, FALSE);
-				
+
 				object = e_cal_component_get_as_string (comp);
 				e_cal_backend_notify_object_created (E_CAL_BACKEND (backend), object);
 				g_free (object);
@@ -2199,11 +2377,19 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 		case ICAL_METHOD_CANCEL:
 			if (cancel_received_object (cbfile, subcomp)) {
 				object = (char *) icalcomponent_as_ical_string (subcomp);
-				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, object);
+				obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
+				if (obj_data)
+					old_object = e_cal_component_get_as_string (obj_data->full_object);
+				else
+					old_object = NULL;
+
+				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, old_object, object);
 
 				/* remove the component from the toplevel VCALENDAR */
 				icalcomponent_remove_component (toplevel_comp, subcomp);
 				icalcomponent_free (subcomp);
+
+				g_free (old_object);
 			}
 			break;
 		default:
@@ -2248,7 +2434,7 @@ e_cal_backend_file_send_objects (ECalBackendSync *backend, EDataCal *cal, const
 
 /* Object initialization function for the file backend */
 static void
-e_cal_backend_file_init (ECalBackendFile *cbfile, ECalBackendFileClass *class)
+e_cal_backend_file_init (ECalBackendFile *cbfile)
 {
 	ECalBackendFilePrivate *priv;
 
@@ -2258,12 +2444,17 @@ e_cal_backend_file_init (ECalBackendFile *cbfile, ECalBackendFileClass *class)
 	priv->uri = NULL;
 	priv->file_name = g_strdup ("calendar.ics");
 	priv->read_only = FALSE;
+	priv->is_dirty = FALSE;
+	priv->dirty_idle_id = 0;
+	priv->idle_save_mutex = g_mutex_new ();
 	priv->icalcomp = NULL;
 	priv->comp_uid_hash = NULL;
 	priv->comp = NULL;
 
 	/* The timezone defaults to UTC. */
 	priv->default_zone = icaltimezone_get_utc_timezone ();
+
+	e_cal_backend_sync_set_lock (E_CAL_BACKEND_SYNC (cbfile), TRUE);
 }
 
 /* Class initialization function for the file backend */
