@@ -60,13 +60,13 @@ CamelProviderConfEntry groupwise_conf_entries[] = {
 	  N_("Check new messages for Junk contents"), "0" },
 	{ CAMEL_PROVIDER_CONF_CHECKBOX, "filter_junk_inbox", "filter_junk",
 	  N_("Only check for Junk messages in the INBOX folder"), "0" },
-	{ CAMEL_PROVIDER_CONF_CHECKBOX, "offline_sync", NULL,
+	{ CAMEL_PROVIDER_CONF_CHECKBOX, "sync_offline", NULL,
 	  N_("Automatically synchronize account locally"), "0" },
 	{ CAMEL_PROVIDER_CONF_SECTION_END },
 
 	/* extra Groupwise  configuration settings */
 	{CAMEL_PROVIDER_CONF_SECTION_START, "soapport", NULL,
-	  N_("Address Book and Calendar") },
+	  N_("SOAP Settings") },
 
 	{ CAMEL_PROVIDER_CONF_ENTRY, "soap_port", NULL,
 	  N_("Post Office Agent SOAP Port:"), "7191" },
@@ -125,21 +125,23 @@ camel_provider_module_init(void)
 {
 	CamelProvider *imap_provider = NULL;
 	CamelException ex = CAMEL_EXCEPTION_INITIALISER;
-	gboolean use_soap_mailer = g_getenv ("SOAP_MAILER") != NULL;
+	gboolean use_imap = g_getenv ("USE_IMAP") != NULL;
 
-	if (!use_soap_mailer)
+	if (use_imap)
 	    imap_provider =  camel_provider_get("imap://", &ex);
 
 	groupwise_provider.url_hash = groupwise_url_hash;
 	groupwise_provider.url_equal = groupwise_url_equal;
 	groupwise_provider.auto_detect = groupwise_auto_detect_cb;
 	groupwise_provider.authtypes = g_list_prepend (groupwise_provider.authtypes, &camel_groupwise_password_authtype);
-	if (use_soap_mailer) {
+	
+	if (use_imap)
+		groupwise_provider.object_types[CAMEL_PROVIDER_STORE] = imap_provider->object_types [CAMEL_PROVIDER_STORE];
+	else 	{
 		groupwise_provider.object_types[CAMEL_PROVIDER_STORE] =  camel_groupwise_store_get_type() ;
 		groupwise_provider.object_types[CAMEL_PROVIDER_TRANSPORT] = camel_groupwise_transport_get_type();
-	} else {
-		groupwise_provider.object_types[CAMEL_PROVIDER_STORE] = imap_provider->object_types [CAMEL_PROVIDER_STORE];
-	}
+	} 
+	
 	camel_provider_register (&groupwise_provider);
 }
 
