@@ -447,16 +447,39 @@ static struct {
 	{ "UIDPLUS",		IMAP_CAPABILITY_UIDPLUS },
 	{ "LITERAL+",		IMAP_CAPABILITY_LITERALPLUS },
 	{ "STARTTLS",           IMAP_CAPABILITY_STARTTLS },
+	{ "XGWEXTENSIONS",      IMAP_CAPABILITY_XGWEXTENSIONS },
+	{ "XGWMOVE",            IMAP_CAPABILITY_XGWMOVE },
 	{ NULL, 0 }
 };
 
+static void
+parse_capability(CamelImapStore *store, char *capa)
+{
+	char *lasts;
+	int i;
+
+	for (capa = strtok_r (capa, " ", &lasts); capa; capa = strtok_r (NULL, " ", &lasts)) {
+		if (!strncmp (capa, "AUTH=", 5)) {
+			g_hash_table_insert (store->authtypes,
+					     g_strdup (capa + 5),
+					     GINT_TO_POINTER (1));
+			continue;
+		}
+		for (i = 0; capabilities[i].name; i++) {
+			if (g_ascii_strcasecmp (capa, capabilities[i].name) == 0) {
+				store->capabilities |= capabilities[i].flag;
+				break;
+			}
+		}
+	}
+}
+
 static gboolean
 imap_get_capability (CamelService *service, CamelException *ex)
 {
 	CamelImapStore *store = CAMEL_IMAP_STORE (service);
 	CamelImapResponse *response;
-	char *result, *capa, *lasts;
-	int i;
+	char *result;
 	
 	CAMEL_SERVICE_ASSERT_LOCKED (store, connect_lock);
 	
@@ -472,23 +495,18 @@ imap_get_capability (CamelService *service, CamelException *ex)
 		return FALSE;
 	
 	/* Skip over "* CAPABILITY ". */
-	capa = result + 13;
-	for (capa = strtok_r (capa, " ", &lasts); capa;
-	     capa = strtok_r (NULL, " ", &lasts)) {
-		if (!strncmp (capa, "AUTH=", 5)) {
-			g_hash_table_insert (store->authtypes,
-					     g_strdup (capa + 5),
-					     GINT_TO_POINTER (1));
-			continue;
-		}
-		for (i = 0; capabilities[i].name; i++) {
-			if (g_ascii_strcasecmp (capa, capabilities[i].name) == 0) {
-				store->capabilities |= capabilities[i].flag;
-				break;
-			}
+	parse_capability(store, result+13);
+	g_free (result);
+
+	/* dunno why the groupwise guys didn't just list this in capabilities */
+	if (store->capabilities & IMAP_CAPABILITY_XGWEXTENSIONS) {
+		/* not critical if this fails */
+		response = camel_imap_command (store, NULL, NULL, "XGWEXTENSIONS");
+		if (response && (result = camel_imap_response_extract (store, response, "XGWEXTENSIONS ", NULL))) {
+			parse_capability(store, result+16);
+			g_free (result);
 		}
 	}
-	g_free (result);
 	
 	imap_set_server_level (store);
 	
@@ -954,7 +972,9 @@ query_auth_types (CamelService *service, CamelException *ex)
 		return NULL;
 	
 	CAMEL_SERVICE_LOCK (store, connect_lock);
-	connected = connect_to_server_wrapper (service, ex);
+	connected = store->istream != NULL && store->connected;
+	if (!connected)
+		connected = connect_to_server_wrapper (service, ex);
 	CAMEL_SERVICE_UNLOCK (store, connect_lock);
 	if (!connected)
 		return NULL;
@@ -997,7 +1017,10 @@ imap_build_folder_info(CamelImapStore *imap_store, const char *folder_name)
 		name = fi->full_name;
 	else
 		name++;
-	fi->name = g_strdup (name);
+	if (!g_ascii_strcasecmp (fi->full_name, "INBOX"))
+		fi->name = g_strdup (_("Inbox"));
+	else
+		fi->name = g_strdup (name);
 	
 	return fi;
 }
@@ -2269,7 +2292,7 @@ create_folder (CamelStore *store, const char *parent_name,
 	}
 	
 	/* FIXME: does not handle unexpected circumstances very well */
-	for (i = 0; i < response->untagged->len; i++) {
+	for (i = 0; i < response->untagged->len && !need_convert; i++) {
 		resp = response->untagged->pdata[i];
 		
 		if (!imap_parse_list_response (imap_store, resp, &flags, NULL, &thisone))
@@ -2278,8 +2301,9 @@ create_folder (CamelStore *store, const char *parent_name,
 		if (strcmp (thisone, parent_name) == 0) {
 			if (flags & CAMEL_FOLDER_NOINFERIORS)
 				need_convert = TRUE;
-			break;
 		}
+
+		g_free(thisone);
 	}
 	
 	camel_imap_response_free (imap_store, response);
@@ -2386,6 +2410,7 @@ parse_list_response_as_folder_info (CamelImapStore *imap_store,
 	/* FIXME: should use imap_build_folder_info, note the differences with param setting tho */
 
 	si = camel_imap_store_summary_add_from_full(imap_store->summary, dir, sep?sep:'/');
+	g_free(dir);
 	if (si == NULL)
 		return NULL;
 
@@ -2394,17 +2419,22 @@ parse_list_response_as_folder_info (CamelImapStore *imap_store,
 		si->info.flags = newflags;
 		camel_store_summary_touch((CamelStoreSummary *)imap_store->summary);
 	}
-	
+
+	flags = (flags & ~CAMEL_FOLDER_SUBSCRIBED) | (si->info.flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED);
+
 	fi = g_new0 (CamelFolderInfo, 1);
-	fi->name = g_strdup(camel_store_info_name(imap_store->summary, si));
 	fi->full_name = g_strdup(camel_store_info_path(imap_store->summary, si));
-	if (!g_ascii_strcasecmp(fi->full_name, "inbox"))
-		flags |= CAMEL_FOLDER_SYSTEM;
+	if (!g_ascii_strcasecmp(fi->full_name, "inbox")) {
+		flags |= CAMEL_FOLDER_SYSTEM|CAMEL_FOLDER_TYPE_INBOX;
+		fi->name = g_strdup (_("Inbox"));
+	} else 
+		fi->name = g_strdup(camel_store_info_name(imap_store->summary, si));
+	
 	/* HACK: some servers report noinferiors for all folders (uw-imapd)
 	   We just translate this into nochildren, and let the imap layer enforce
 	   it.  See create folder */
 	if (flags & CAMEL_FOLDER_NOINFERIORS)
-		flags = (fi->flags & ~CAMEL_FOLDER_NOINFERIORS) | CAMEL_FOLDER_NOCHILDREN;
+		flags = (flags & ~CAMEL_FOLDER_NOINFERIORS) | CAMEL_FOLDER_NOCHILDREN;
 	fi->flags = flags;
 	
 	url = camel_url_new (imap_store->base_url, NULL);
@@ -2571,11 +2601,12 @@ get_folders_online (CamelImapStore *imap_store, const char *pattern,
 			continue;
 
 		if (imap_match_pattern(imap_store->dir_sep, pattern, camel_imap_store_info_full_name(imap_store->summary, si))) {
-			if (g_hash_table_lookup(present, camel_store_info_path(imap_store->summary, si)) != NULL) {
+			if ((fi = g_hash_table_lookup(present, camel_store_info_path(imap_store->summary, si))) != NULL) {
 				if (lsub && (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) == 0) {
 					si->flags |= CAMEL_STORE_INFO_FOLDER_SUBSCRIBED;
 					camel_store_summary_touch((CamelStoreSummary *)imap_store->summary);
 				}
+				fi->flags = (fi->flags & ~CAMEL_FOLDER_SUBSCRIBED) | (si->flags & CAMEL_FOLDER_SUBSCRIBED);
 			} else {
 				if (lsub) {
 					if (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED) {
