@@ -48,6 +48,10 @@ struct _EGwConnectionPrivate {
 	char *user_email;
 	char *user_uuid;
 	char *version;
+	char *server_time ;
+	GHashTable *categories_by_name;
+	GHashTable *categories_by_id;
+	GList *book_list;
 	EGwSendOptions *opts;
 	GMutex *reauth_mutex;
 };
@@ -274,11 +278,37 @@ e_gw_connection_dispose (GObject *object)
 			g_mutex_free (priv->reauth_mutex);
 			priv->reauth_mutex = NULL;
 		}
-
+	
+		if (priv->categories_by_id) {
+			g_hash_table_destroy (priv->categories_by_id);
+			priv->categories_by_id = NULL;
+		}	
+		
+		if (priv->categories_by_name) {
+			g_hash_table_destroy (priv->categories_by_name);
+			priv->categories_by_name = NULL;
+		}
+	
+		if (priv->book_list) {
+			g_list_foreach (priv->book_list, (GFunc) g_object_unref, NULL);
+			g_list_free (priv->book_list);
+			priv->book_list = NULL;
+		}
+		
 		if (priv->opts) {
 			g_object_unref (priv->opts);
 			priv->opts = NULL;
 		}
+		
+		if (priv->version) {
+			g_free (priv->version) ;
+			priv->opts = NULL ;
+		}
+
+		if (priv->server_time) {
+			g_free (priv->server_time) ;
+			priv->server_time = NULL ;
+		}
 	}
 
 	if (parent_class->dispose)
@@ -326,6 +356,9 @@ e_gw_connection_init (EGwConnection *cnc, EGwConnectionClass *klass)
 	/* create the SoupSession for this connection */
 	priv->soup_session = soup_session_sync_new ();
 	priv->reauth_mutex = g_mutex_new ();
+	priv->categories_by_id = NULL;
+	priv->categories_by_name = NULL;
+	priv->book_list = NULL;
 	priv->opts = NULL;
 }
 
@@ -460,24 +493,24 @@ e_gw_connection_new (const char *uri, const char *username, const char *password
 	
 	if (param) {
 		SoupSoapParameter *subparam;
-		const char *param_value;
+		char *param_value;
 
 		subparam = soup_soap_parameter_get_first_child_by_name (param, "email");
 		if (subparam) {
 			param_value = soup_soap_parameter_get_string_value (subparam);
-			cnc->priv->user_email  = g_strdup (param_value);
+			cnc->priv->user_email  = param_value;
 		}
 
 		subparam = soup_soap_parameter_get_first_child_by_name (param, "name");
 		if (subparam) {
 			param_value = soup_soap_parameter_get_string_value (subparam);
-			cnc->priv->user_name = g_strdup (param_value);
+			cnc->priv->user_name = param_value;
 		}
 
 		subparam = soup_soap_parameter_get_first_child_by_name (param, "uuid");
 		if (subparam) {
 			param_value = soup_soap_parameter_get_string_value (subparam);
-			cnc->priv->user_uuid = g_strdup (param_value);
+			cnc->priv->user_uuid = param_value;
 		}
 	}
 
@@ -489,6 +522,9 @@ e_gw_connection_new (const char *uri, const char *username, const char *password
 	} else
 	       	cnc->priv->version = NULL;	
 
+	param = soup_soap_response_get_first_parameter_by_name (response, "serverUTCTime");
+	if (param) 
+		cnc->priv->server_time = soup_soap_parameter_get_string_value (param);
 
 	/* add the connection to the loaded_connections hash table */
 	hash_key = g_strdup_printf ("%s:%s@%s",
@@ -1342,6 +1378,14 @@ e_gw_connection_get_user_uuid (EGwConnection *cnc)
 	return (const char *) cnc->priv->user_uuid;
 }
 
+const char * 
+e_gw_connection_get_server_time (EGwConnection *cnc)
+{
+	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), NULL) ;
+
+	return (const char *) cnc->priv->server_time ;
+}
+
 static time_t
 timet_from_string (const char *str)
 {
@@ -1467,16 +1511,28 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	SoupSoapMessage *msg;
 	SoupSoapResponse *response;
         EGwConnectionStatus status;
+	EGwConnectionPrivate *priv;
 	SoupSoapParameter *param;
 	SoupSoapParameter *type_param;
 	char *value;
-	
+	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;
+
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_UNKNOWN);
 	g_return_val_if_fail (container_list != NULL, E_GW_CONNECTION_STATUS_UNKNOWN);
 
+	priv = cnc->priv;
+	g_static_mutex_lock (&connecting);
+
+	if (priv->book_list) {
+		*container_list = priv->book_list;
+		g_static_mutex_unlock (&connecting);
+		return E_GW_CONNECTION_STATUS_OK;
+	}
+
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getAddressBookListRequest");
         if (!msg) {
                 g_warning (G_STRLOC ": Could not build SOAP message");
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_UNKNOWN;
         }
 
@@ -1486,6 +1542,7 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
@@ -1496,6 +1553,7 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 		if (status == E_GW_CONNECTION_STATUS_INVALID_CONNECTION)
 			reauthenticate (cnc);
                 g_object_unref (response);
+		g_static_mutex_unlock (&connecting);
                 return status;
         }
 	
@@ -1503,6 +1561,7 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	param = soup_soap_response_get_first_parameter_by_name (response, "books");	
         if (!param) {
                 g_object_unref (response);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         } else {
 		SoupSoapParameter *subparam;
@@ -1513,7 +1572,7 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 				       
 			container = e_gw_container_new_from_soap_parameter (subparam);
 			if (container) {
-				*container_list = g_list_append (*container_list, container);
+				priv->book_list = g_list_append (priv->book_list, container);
 				type_param = soup_soap_parameter_get_first_child_by_name (subparam, "isPersonal");
 				value = NULL;
 				if (type_param)
@@ -1538,6 +1597,8 @@ e_gw_connection_get_address_book_list (EGwConnection *cnc, GList **container_lis
 	}
 
 	g_object_unref (response);
+	*container_list = priv->book_list;
+	g_static_mutex_unlock (&connecting);
 
         return status;
 }
@@ -1554,7 +1615,6 @@ e_gw_connection_get_address_book_id ( EGwConnection *cnc, char *book_name, char*
 
 	status = e_gw_connection_get_address_book_list (cnc, &container_list);
         if (status != E_GW_CONNECTION_STATUS_OK) {
-		e_gw_connection_free_container_list (container_list);
                 return status;
         }
 
@@ -1569,8 +1629,6 @@ e_gw_connection_get_address_book_id ( EGwConnection *cnc, char *book_name, char*
 		}
 	}
 
-	e_gw_connection_free_container_list (container_list);
-
 	return status;
 
 }
@@ -1636,14 +1694,19 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
 	EGwConnectionStatus status;
 	SoupSoapParameter *param;
 	EGwConnectionPrivate *priv;
+	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;	
+
 
 	g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
 
 	priv = cnc->priv;
 
+	g_static_mutex_lock (&connecting);
+
 	if (priv->opts) {
 		g_object_ref (priv->opts);
 		*opts = priv->opts;
+		g_static_mutex_unlock (&connecting);
 		
 		return E_GW_CONNECTION_STATUS_OK;
 	}
@@ -1651,6 +1714,7 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getSettingsRequest");
 	if (!msg) {
                 g_warning (G_STRLOC ": Could not build SOAP message");
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_UNKNOWN;
         }
        	e_gw_message_write_footer (msg);
@@ -1659,6 +1723,7 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
         response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
@@ -1668,6 +1733,7 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
 			reauthenticate (cnc);
 		g_object_unref (response);
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
 		return status;
 	}
 	
@@ -1675,6 +1741,7 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
         if (!param) {
                 g_object_unref (response);
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         } else 
 		priv->opts = e_gw_sendoptions_new_from_soap_parameter (param);
@@ -1683,24 +1750,38 @@ e_gw_connection_get_settings (EGwConnection *cnc, EGwSendOptions **opts)
 	*opts = priv->opts;
 	g_object_unref (response);
         g_object_unref (msg);
+	g_static_mutex_unlock (&connecting);
 	
 	return E_GW_CONNECTION_STATUS_OK;
 }
 
 EGwConnectionStatus 
-e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id, GHashTable *categories_by_name)
+e_gw_connection_get_categories (EGwConnection *cnc, GHashTable **categories_by_id, GHashTable **categories_by_name)
 {
 	SoupSoapMessage *msg;
         SoupSoapResponse *response;
         EGwConnectionStatus status;
+	EGwConnectionPrivate *priv;
         SoupSoapParameter *param, *subparam, *second_level_child;
 	char *id, *name;
         g_return_val_if_fail (E_IS_GW_CONNECTION (cnc), E_GW_CONNECTION_STATUS_INVALID_OBJECT);
-	
+	static GStaticMutex connecting = G_STATIC_MUTEX_INIT;
+
+	priv = cnc->priv;
+	g_static_mutex_lock (&connecting);
+
+	if (priv->categories_by_id && priv->categories_by_name) {
+		*categories_by_id = priv->categories_by_id;
+		*categories_by_name = priv->categories_by_name;
+		g_static_mutex_unlock (&connecting);
+		return E_GW_CONNECTION_STATUS_OK;
+	}
+
 	/* build the SOAP message */
         msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getCategoryListRequest");
         if (!msg) {
                 g_warning (G_STRLOC ": Could not build SOAP message");
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_UNKNOWN;
         }
        	e_gw_message_write_footer (msg);
@@ -1709,6 +1790,7 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
         response = e_gw_connection_send_message (cnc, msg);
         if (!response) {
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_NO_RESPONSE;
         }
 
@@ -1718,6 +1800,7 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
 			reauthenticate (cnc);
 		g_object_unref (response);
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
 		return status;
 	}
 
@@ -1726,9 +1809,13 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
         if (!param) {
                 g_object_unref (response);
                 g_object_unref (msg);
+		g_static_mutex_unlock (&connecting);
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         }
 	
+	priv->categories_by_id = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	priv->categories_by_name = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+
 	for (subparam = soup_soap_parameter_get_first_child_by_name (param, "category");
 	     subparam != NULL;
 	     subparam = soup_soap_parameter_get_next_child_by_name (subparam, "category")) {
@@ -1744,9 +1831,9 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
 			g_free (id);
 			id = components[0];
 			if (categories_by_id) 
-				g_hash_table_insert (categories_by_id, g_strdup (id), g_strdup (name));
+				g_hash_table_insert (priv->categories_by_id, g_strdup (id), g_strdup (name));
 			if (categories_by_name) 
-				g_hash_table_insert (categories_by_name, g_strdup (name), g_strdup (id));
+				g_hash_table_insert (priv->categories_by_name, g_strdup (name), g_strdup (id));
 			g_strfreev (components);
 			g_free (name);
 		}
@@ -1756,10 +1843,11 @@ e_gw_connection_get_categories (EGwConnection *cnc, GHashTable *categories_by_id
 	/* free memory */
         g_object_unref (response);
 	g_object_unref (msg);
+	*categories_by_id = priv->categories_by_id;
+	*categories_by_name = priv->categories_by_name;
+	g_static_mutex_unlock (&connecting);
 
         return E_GW_CONNECTION_STATUS_OK;
-
-
 }
 
 EGwConnectionStatus 
@@ -1972,7 +2060,7 @@ e_gw_connection_position_cursor (EGwConnection *cnc, const char *container, int
 }
 
 EGwConnectionStatus
-e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int cursor, gboolean forward, int count, GList **item_list)
+e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int cursor, gboolean forward, int count, const char *cursor_seek, GList **item_list)
 {
 	SoupSoapMessage *msg;
 	SoupSoapResponse *response;
@@ -1986,6 +2074,7 @@ e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int curs
 	e_gw_message_write_int_parameter (msg, "cursor", NULL, cursor);
 	/* there is problem in read curosr if you set this, uncomment after the problem 
 	   is fixed in server */
+	e_gw_message_write_string_parameter (msg, "position", NULL, cursor_seek);
 	e_gw_message_write_string_parameter (msg, "forward", NULL, forward ? "true": "false");
 	e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	e_gw_message_write_int_parameter (msg, "count", NULL, count);
@@ -2031,7 +2120,7 @@ e_gw_connection_read_cursor (EGwConnection *cnc, const char *container, int curs
         return E_GW_CONNECTION_STATUS_OK;
 }
 
-EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, const char *container, const char *view, const char *start_date, const char *message_list, const char *item_types, const char *item_sources, int count, GSList **item_list)
+EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, const char *container, const char *view, char **start_date, const char *message_list, const char *item_types, const char *item_sources, int count, GSList **item_list)
 {
 	SoupSoapMessage *msg;
 	SoupSoapResponse *response;
@@ -2043,8 +2132,8 @@ EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, cons
 
 	msg = e_gw_message_new_with_header (cnc->priv->uri, cnc->priv->session_id, "getQuickMessagesRequest");
 	e_gw_message_write_string_parameter (msg, "list", NULL, message_list);
-	if (start_date)
-		e_gw_message_write_string_parameter (msg, "startDate", NULL, start_date);
+	if (start_date && *start_date)
+		e_gw_message_write_string_parameter (msg, "startDate", NULL, *start_date);
 	if (container)
 		e_gw_message_write_string_parameter (msg, "container", NULL, container);
 	if (item_types) 
@@ -2081,7 +2170,20 @@ EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, cons
                 g_object_unref (msg);
                 return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
         }
+	if (start_date && *start_date && !strcmp (message_list, "New")) {
+		subparam = soup_soap_response_get_first_parameter_by_name (response, "startDate");
+		if (subparam) {
+			char *date;
 
+			date = soup_soap_parameter_get_string_value (subparam);
+			if (date)
+				g_free (*start_date), *start_date = NULL, *start_date = date;
+			else 
+				return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+		} else 
+			return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+	}
+	
 	if (!strcmp (message_list, "All")) { 
 		/* We are  interested only in getting the ids */
 		for (subparam = soup_soap_parameter_get_first_child_by_name (param, "item");
@@ -2092,9 +2194,12 @@ EGwConnectionStatus e_gw_connection_get_quick_messages (EGwConnection *cnc, cons
 			
 			param_id = soup_soap_parameter_get_first_child_by_name (subparam, "iCalId");
 			if (!param_id) {
-				g_object_unref (response);
-		                g_object_unref (msg);
-                		return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+				param_id = soup_soap_parameter_get_first_child_by_name (subparam, "id");
+				if (!param_id) {
+					g_object_unref (response);
+					g_object_unref (msg);
+					return E_GW_CONNECTION_STATUS_INVALID_RESPONSE;
+				}
 			}
 		     
 			id = g_strdup (soup_soap_parameter_get_string_value (param_id));
