@@ -549,7 +549,7 @@ set_recipient_list_from_soap_parameter (EGwItem *item, SoupSoapParameter *param)
 
 			recipient->status_enabled = TRUE;
        			if ( (temp_param = soup_soap_parameter_get_first_child_by_name (subparam, "deleted")) ) {
-				recipient->status = E_GW_ITEM_STAT_DECLINED;
+				recipient->status = E_GW_ITEM_STAT_DELETED;
 				value = soup_soap_parameter_get_string_value (temp_param);
 				formatted_date = e_gw_connection_format_date_string (value);
 				recipient->deleted_date = g_strdup (formatted_date);
@@ -682,6 +682,7 @@ set_sendoptions_from_soap_parameter (EGwItem *item, SoupSoapParameter *param)
 		       value = soup_soap_parameter_get_property (subparam, "autoDelete");
 		       if (value && !g_ascii_strcasecmp (value, "1"))
 		      		priv->autodelete = TRUE; 
+		       g_free (value), value = NULL;
 	       }	       
 	}
 
@@ -1020,10 +1021,14 @@ set_contact_fields_from_soap_parameter (EGwItem *item, SoupSoapParameter *param)
 		}
 		for ( temp = soup_soap_parameter_get_first_child (subparam); temp != NULL; temp = soup_soap_parameter_get_next_child (temp)) {
 			value = soup_soap_parameter_get_string_value (temp);
-			if (value && (!primary_email ||  !g_str_equal (primary_email, value)))
-				item->priv->email_list = g_list_append (item->priv->email_list, value);
+			if (value) {
+				if (!primary_email ||  !g_str_equal (primary_email, value))
+					item->priv->email_list = g_list_append (item->priv->email_list, value);
+				else
+					g_free (value);
+			}
 		}
-		g_free (value);
+		g_free (primary_email);
 	}
 	
 	subparam =  soup_soap_parameter_get_first_child_by_name(param, "imList");
@@ -1116,8 +1121,8 @@ set_contact_fields_from_soap_parameter (EGwItem *item, SoupSoapParameter *param)
 			value = soup_soap_parameter_get_property(temp, "type");
 			if (value)
 				g_hash_table_insert (item->priv->addresses, value, address);
-			 			
-			
+			else
+				free_postal_address (address);
 		}
 		
 	}
@@ -1633,7 +1638,7 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 				else if (!strcmp (status_name, "declined"))
 					item->priv->self_status |= E_GW_ITEM_STAT_DECLINED;
 				else if (!strcmp (status_name, "deleted"))
-					item->priv->self_status |= E_GW_ITEM_STAT_DECLINED;
+					item->priv->self_status |= E_GW_ITEM_STAT_DELETED;
 				else if (!strcmp (status_name, "read")) 
 					item->priv->self_status |= E_GW_ITEM_STAT_READ;
 				else if (!strcmp (status_name, "opened"))
@@ -1729,8 +1734,8 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 		} else if (!g_ascii_strcasecmp (name, "to")) {
 			char *to ;
 			to = soup_soap_parameter_get_string_value (child) ;
-			e_gw_item_set_to (item,to) ;
-
+			e_gw_item_set_to (item, to);
+			g_free (to);
 		} else if (!g_ascii_strcasecmp (name, "iCalId"))
 			item->priv->icalid = soup_soap_parameter_get_string_value (child);
 
@@ -1779,8 +1784,15 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 				SoupSoapParameter *temp ;
 				EGwItemAttachment *attach = g_new0 (EGwItemAttachment, 1) ;
 				temp = soup_soap_parameter_get_first_child_by_name (attachment_param, "id") ;
-				if (temp)	
+				if (temp) {
+					char *ref = NULL;
 					attach->id = soup_soap_parameter_get_string_value (temp) ;
+					ref =  soup_soap_parameter_get_property (temp, "itemReference");
+					if (ref) {
+						attach->contentType = g_strdup_printf ("gw:%s", ref);
+						g_free (ref);
+					}	
+				}
 				
 				temp = soup_soap_parameter_get_first_child_by_name (attachment_param, "name") ;
 				if (temp)
@@ -1806,7 +1818,11 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 			item->priv->place = soup_soap_parameter_get_string_value (child);
 
 		else if (!g_ascii_strcasecmp (name, "taskPriority")) {
-			e_gw_item_set_task_priority (item, soup_soap_parameter_get_string_value (child));
+			char *priority;
+
+			priority = soup_soap_parameter_get_string_value (child);
+			e_gw_item_set_task_priority (item, priority);
+			g_free (priority);
 		}
 
 		else if (!g_ascii_strcasecmp (name, "startDate")) {
@@ -1828,7 +1844,7 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 		}
 			
 		else if (!g_ascii_strcasecmp (name, "alarm")) {
-			const char *enabled;
+			char *enabled;
 			enabled = soup_soap_parameter_get_property (child, "enabled");
 			if (!g_ascii_strcasecmp (enabled, "true") ) {
 				char *value;
@@ -1836,6 +1852,7 @@ e_gw_item_new_from_soap_parameter (const char *email, const char *container, Sou
 				/* convert it into integer */
 				item->priv->trigger = atoi (value);
 				g_free (value);
+				g_free (enabled);
 			}
 		}
 		
@@ -2457,6 +2474,24 @@ e_gw_item_set_source (EGwItem *item, char *source)
 	item->priv->source = g_strdup (source);
 }
 
+void
+e_gw_item_set_content_type (EGwItem *item, const char *content_type)
+{
+	g_return_if_fail (E_IS_GW_ITEM (item));
+
+	if (item->priv->content_type)
+		g_free (item->priv->content_type);
+	item->priv->content_type= g_strdup (content_type);
+}
+
+char *
+e_gw_item_get_content_type (EGwItem *item)
+{
+	g_return_val_if_fail (E_IS_GW_ITEM (item), NULL);
+
+	return item->priv->content_type ;
+}
+
 static void
 add_return_notification (SoupSoapMessage *msg, char *option, EGwItemReturnNotify value)
 {
@@ -2533,6 +2568,8 @@ add_distribution_to_soap_message (EGwItem *item, SoupSoapMessage *msg)
 			dist_type = "TO";
 		else if (recipient->type == E_GW_ITEM_RECIPIENT_CC)
 			dist_type = "CC";
+		else if (recipient->type == E_GW_ITEM_RECIPIENT_BC)
+			dist_type = "BC";
 		else 
 			dist_type ="";
 		e_gw_message_write_string_parameter (msg, "distType", NULL, dist_type);
@@ -2762,7 +2799,7 @@ e_gw_item_append_to_soap_message (EGwItem *item, SoupSoapMessage *msg)
 		if (item->priv->container)
 			e_gw_message_write_string_parameter (msg, "container", NULL, item->priv->container);
 		if (item->priv->source)
-			e_gw_message_write_string_parameter (msg, "source", NULL, item->priv->source );
+			e_gw_message_write_string_parameter (msg, "source", NULL, item->priv->source);
 		
 		/*message*/
 		soup_soap_message_start_element (msg, "message", NULL, NULL);
@@ -2772,9 +2809,12 @@ e_gw_item_append_to_soap_message (EGwItem *item, SoupSoapMessage *msg)
 
 			str = soup_base64_encode (priv->message, strlen (priv->message));
 			str_len = g_strdup_printf ("%d", strlen (str));
+			soup_soap_message_start_element (msg, "part", NULL, NULL);
 			soup_soap_message_add_attribute (msg, "length", str_len, NULL, NULL);
+			soup_soap_message_add_attribute (msg, "contentType", priv->content_type, NULL, NULL);
 			g_free (str_len);
 			soup_soap_message_write_string (msg, str);
+			soup_soap_message_end_element (msg);
 			g_free (str);
 		} else {
 			soup_soap_message_add_attribute (msg, "length", "0", NULL, NULL);
